angular.module('starter.controllers', [])
    .controller('SignInCtrl', function($scope, $state, $stateParams, $cordovaDialogs, $ionicHistory, $cordovaProgress, SureSpotApiService) {
        $scope.state = {
            email: '',
            password: '',
            userId: null,
        };

        $scope.goSignUp = function () {
            $state.go('signup');
        };

        $scope.goFgtPwd = function () {
            $state.go('forgot-password');
        };

        $scope.handleSignIn = function() {
            console.log($scope.state);
            $cordovaProgress.showSimple(true);

            SureSpotApiService.Authentication_SignIn($scope.state).then(function(data) {
                $cordovaProgress.hide();

                var USER_NOT_FOUND = (data === 'No validation found') ||
                                     (data === 'No UserName or Email found');

                if (USER_NOT_FOUND) {
                    $cordovaDialogs.alert('Wrong email/password!', 'Warning', 'Close');
                    return;
                }
                
                $scope.state.userId = data;

                // save user to localStorage
                localStorage.setItem('user', JSON.stringify({
                    email: $scope.state.email,
                    password: $scope.state.password,
                    userId: $scope.state.userId,
                }));

                $ionicHistory.clearHistory();
                $state.go('tab.search');
            }, function(err) {
                $cordovaProgress.hide();
                $cordovaDialogs.alert('Please check your network connection.', 'Warning', 'Close');
            });
        };
    })

    .controller('SignUpCtrl', function($scope, $state, $stateParams, $cordovaDialogs, $cordovaProgress, SureSpotApiService) {
        $scope.state = {
            firstName: '',
            lastName: '',
            email: '',
            password: '',
        };

        $scope.handleSignUp = function() {
            console.log($scope.state);
            $cordovaProgress.showSimple(true);

            SureSpotApiService.Authentication_SignUp($scope.state).then(function(data) {
                $cordovaProgress.hide();

                var USER_EXISTS = ('UserName already exists' === data) ||
                                  ('username exists' === data);

                if (USER_EXISTS) {
                    $cordovaDialogs.alert('Please try another email!', 'Warning', 'Close');
                    return;
                }

                $cordovaDialogs.alert('Activation email will be sent shortly. Please check you email inbox, and click on link to activate your account.', 'Warning', 'Close');

                $state.go('signin');
            }, function(err) {
                $cordovaProgress.hide();
                $cordovaDialogs.alert('Please check your network connection.', 'Warning', 'Close');
            });
        };

        $scope.handleSignIn = function() {
            $state.go('signin');
        };
    })

    .controller('FgtPwdCtrl', function($scope, $state, $stateParams, $cordovaProgress, SureSpotApiService) {
        $scope.state = {
            email: '',
            password: '', // ?
        };

        $scope.handleSend = function() {
            $cordovaProgress.showSimple(true);

            SureSpotApiService.Authentication_LostPassword($scope.state).then(function(data) {
                $cordovaProgress.hide();
                $state.go('signin');
            }, function(err) {
                $cordovaProgress.hide();
                $cordovaDialogs.alert('Please check your network connection.', 'Warning', 'Close');
            });
        };

        $scope.handleSignIn = function() {
            $state.go('signin');
        };
    })

    .controller('MapCtrl', function($scope, $state, $stateParams, $cordovaProgress, $cordovaGeolocation, SureSpotApiService) {
        $scope.state = {
            search: '',
            parkingLots: [],
            watch: null,
        };

        $scope.center = {
            lat: 0,
            lng: 0,
            zoom: 11
        };

        $scope.markers = {
            current: {
                lat: 0,
                lng: 0,
                focus: true,
                draggable: true
            }
        };

        $scope.defaults = {
            tileLayerOptions: {
                opacity: 0.9,
                detectRetina: true,
                reuseTiles: true,
            },
            tileLayer: 'http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
            tileLayerOptions: {
                attribution: 'OpenStreetMap'
            },
        };

        $scope.$on('$ionicView.enter', function() {
            var posOptions = {
                timeout: 30000,
                enableHighAccuracy: true
            };
            
            $cordovaGeolocation
                .getCurrentPosition(posOptions)
                .then(function (position) {
                    console.log('position:', position);

                    // center map to current position
                    if ($scope.center.lat == $scope.center.lng == 0) {
                        $scope.center.lat = position.coords.latitude;
                        $scope.center.lng = position.coords.longitude;
                    }

                    $scope.markers.current.lat = position.coords.latitude;
                    $scope.markers.current.lng = position.coords.longitude;

                    // watch changes
                    $scope.state.watch = $cordovaGeolocation
                        .watchPosition(posOptions);

                    $scope.state.watch
                        .then(
                            null,
                            function(err) {
                                console.log(err);
                            },
                            function (position) {
                                console.log('position:', position);
                                $scope.markers.current.lat = position.coords.latitude;
                                $scope.markers.current.lng = position.coords.longitude;
                            }
                        );
                }, function(err) {
                    console.log(err);
                });
        });

        $scope.$on('$ionicView.leave', function() {
            if ($scope.state.watch) {
                $scope.state.watch.clearWatch();
                $scope.state.watch = null;
            }
        });

        $scope.loadParkingLots = function() {
            $cordovaProgress.showSimple(true);

            SureSpotApiService.Lots_GetParkingLotsList($scope.state.search).then(function(data) {
                console.log(data);
                $cordovaProgress.hide();
                $scope.state.parkingLots = data;
                
                $scope.markers = {
                    current: $scope.markers.current
                };

                _.each($scope.state.parkingLots, function(parkingLot, parkingLotIndex) {
                    _.each(parkingLot.Zones, function(zone, zoneIndex) {
                        var geo = _.split(zone.geotags, ',');
                        var lat = parseFloat(geo[0]);
                        var lng = parseFloat(geo[1]);
                        
                        $scope.markers[zone.car_park_zone] = {
                            lat: lat,
                            lng: lng,
                            // focus: true,
                            icon: {
                                iconUrl: 'img/marker-violet.png',
                                iconSize: [25, 41],
                                iconAnchor: [13, 41],
                                popupAnchor: [0, -41],
                                shadowSize: [0, 0],
                                shadowAnchor: [0, 0]
                            },
                            message: (
                                '<div>' +
                                parkingLot.car_park +
                                '<br>' +
                                zone.car_park_zone +
                                '<br>' +
                                '<button class="button ss-btn" ng-click="goParkingByIndex(' + parkingLotIndex + ', ' + zoneIndex + ');">Reserve</button>' +
                                '</div>'
                            ),
                            getMessageScope: function() {
                                return $scope;
                            }
                        };
                    });
                });
            }, function(err) {
                $cordovaProgress.hide();
                $cordovaDialogs.alert('Please check your network connection.', 'Warning', 'Close');
            });
        };
        
        setTimeout($scope.loadParkingLots, 1000);

        $scope.goParkingByIndex = function(parkingLotIndex, zoneIndex) {
            var parkingLot = $scope.state.parkingLots[parkingLotIndex];
            var zone = parkingLot.Zones[zoneIndex];

            $state.go('tab.parking-lot', {
                parkingLot: parkingLot,
                zone: zone,
            });
        };
    })

    .controller('SearchCtrl', function($scope, $state, $stateParams, $cordovaProgress, SureSpotApiService) {
        $scope.state = {
            search: '',
            parkingLots: [],
        };

        // show all parking lots
        $scope.loadParkingLots = function() {
            $cordovaProgress.showSimple(true);

            SureSpotApiService.Lots_GetParkingLotsList($scope.state.search).then(function(data) {
                console.log(data);
                $cordovaProgress.hide();
                $scope.state.parkingLots = data;
            }, function(err) {
                $cordovaProgress.hide();
                $cordovaDialogs.alert('Please check your network connection.', 'Warning', 'Close');
            });
        };
        
        setTimeout($scope.loadParkingLots, 1000);

        $scope.handleSearchChange = function() {
            $cordovaProgress.showSimple(true);

            SureSpotApiService.Lots_GetParkingLotsList($scope.state.search).then(function(data) {
                console.log(data);
                $cordovaProgress.hide();
                $scope.state.parkingLots = data;
            }, function(err) {
                $cordovaProgress.hide();
                $cordovaDialogs.alert('Please check your network connection.', 'Warning', 'Close');
            });
        };

        $scope.goParking = function(parkingLot) {
            $state.go('tab.parking-lot', {
                parkingLot: parkingLot,
            });
        };

        // $scope.goMap = function(parkingLot) {
        //     $state.go('tab.parking-map', {
        //         parkingLot: parkingLot,
        //     });
        // };
    })

    .controller('ParkingLotCtrl', function($scope, $state, $stateParams, $cordovaDialogs, $cordovaProgress, SureSpotApiService) {
        $scope.state = {
            parkingLot: $stateParams.parkingLot,
            zone: $stateParams.zone || null,
            datetimeFrom: moment().toDate(),
            datetimeTo: moment().add(1, 'hour').toDate(),
        };

        $scope.goReserve = function(parkingLot, zone, datetimeFrom, datetimeTo) {
            if (zone === null) {
                $cordovaDialogs.alert('Please choose area!', 'Warning', 'Close');
                return;
            }

            var _user = localStorage.getItem('user');
            var user = JSON.parse(_user);
            var datetimeFrom = moment($scope.state.datetimeFrom).format('YYYY-MM-DD HH:mm:ss');
            var datetimeTo = moment($scope.state.datetimeTo).format('YYYY-MM-DD HH:mm:ss');

            $cordovaProgress.showSimple(true);

            SureSpotApiService.Reservations_AddPrepay(user.userId, zone.car_park_zone_id, datetimeFrom, datetimeTo).then(function(data) {
                console.log(data);
                $cordovaProgress.hide();
                var prepay = data;

                $state.go('tab.checkout', {
                    parkingLot: parkingLot,
                    zone: zone,
                    datetimeFrom: datetimeFrom,
                    datetimeTo: datetimeTo,
                    prepay: prepay,
                });
            }, function(err) {
                $cordovaProgress.hide();
                $cordovaDialogs.alert('Please check your network connection.', 'Warning', 'Close');
            });
        };

        $scope.goCancel = function() {
            $state.go('tab.search');
        };
    })

    .controller('ParkingMapCtrl', function($scope, $state, $stateParams, $cordovaProgress, SureSpotApiService) {
        $scope.state = {
            parkingLot: $stateParams.parkingLot,
        };
    })

    .controller('CheckoutCtrl', function($scope, $state, $stateParams, $cordovaDialogs, $cordovaProgress, SureSpotApiService) {
        $scope.state = {
            parkingLot: $stateParams.parkingLot,
            zone: $stateParams.zone,
            datetimeFrom: $stateParams.datetimeFrom,
            datetimeTo: $stateParams.datetimeTo,
            prepay: $stateParams.prepay,
        };

        $scope.pay = function(parkingLot, zone, datetimeFrom, datetimeTo) {
            // load card
            var _card = localStorage.getItem('card');

            if (!_card) {
                $cordovaDialogs.alert('Please enter valid card in More->Card!', 'Warning', 'Close');
                return;
            }

            if (_card) {
                var card = JSON.parse(_card);
                $scope.state.card = card;
            }

            var cardNumber = $scope.state.card.number.split(' ').join('');
            var cardName = $scope.state.card.name.split(' ').join('/');
            var cardExpireDate = $scope.state.card.expiry.slice(-2) + $scope.state.card.expiry.slice(0, 2);
            var serviceCode = '101';
            // var cardCVC = $scope.state.card.cvc;
            var cardCVC = '0000000000000000';

            var magStrip = (
                '%' +
                'B' +
                cardNumber +
                '^' +
                cardName +
                '^' +
                cardExpireDate +
                serviceCode +
                cardCVC +
                '?'
            );

            console.log('magStrip:', magStrip);

            // encrypt
            var b64ciphertextReservationId = SureSpotApiService.encrypt('' + $scope.state.prepay.ReservationId, 'Sure Spot');
            var b64ciphertextPrepayRate = SureSpotApiService.encrypt($scope.state.prepay.PrepayRate, 'Sure Spot');
            var b64ciphertextMagStrip = SureSpotApiService.encrypt(magStrip, 'Sure Spot');
            
            $cordovaProgress.showSimple(true);

            SureSpotApiService.Payments_PayReservation(b64ciphertextReservationId, b64ciphertextPrepayRate, b64ciphertextMagStrip).then(function(data) {
                console.log(data);
                $cordovaProgress.hide();

                var ERROR = (data.toLowerCase().slice(0, 'error:'.length) === 'error:');
                var SUCCESS = (data.toLowerCase().slice(0, 'success'.length) === 'success') || (data.toLowerCase().slice(0, 'sucess'.length) === 'sucess');
                var DECLINED = (data.toLowerCase().slice(0, 'declined'.length) === 'declined');
                
                if (ERROR) {
                    $cordovaDialogs.alert('Payment was unsuccessful! Reason: ' + data.slice('error:'.length), 'Error', 'Close');
                    return;
                }

                if (DECLINED) {
                    $cordovaDialogs.alert('Payment was unsuccessful! Reason: declined.', 'Error', 'Close');
                }

                if (SUCCESS) {
                    // save reservation
                    var _reservations = localStorage.getItem('reservations');
                    var reservations;

                    if (_reservations) {
                        reservations = JSON.parse(_reservations);
                    } else {
                        reservations = [];
                    }
                    
                    var reservation = {
                        id: $scope.state.prepay.ReservationId,
                        status: true,
                        from: datetimeFrom,
                        to: datetimeTo,
                        parkingLot: $scope.state.parkingLot,
                        zone: $scope.state.zone,
                    };

                    reservations.push(reservation);
                    _reservations = JSON.stringify(reservations);
                    localStorage.setItem('reservations', _reservations);

                    $state.go('tab.reservations');
                }
            }, function(err) {
                $cordovaProgress.hide();
                $cordovaDialogs.alert('Please check your network connection.', 'Warning', 'Close');
            });
        };
    })

    .controller('EventsCtrl', function($scope, $state, $stateParams, $cordovaProgress, SureSpotApiService) {
        
    })

    .controller('ReservationsCtrl', function($scope, $state, $stateParams, $cordovaProgress, SureSpotApiService) {
        $scope.state = {
            reservations: [],
        };

        var _user = localStorage.getItem('user');
        var user = JSON.parse(_user);

        function textToBase64Barcode(text) {
            var canvas = document.createElement("canvas");
            JsBarcode(canvas, text, {format: "CODE128"});
            return canvas.toDataURL("image/png");
        }

        $scope.loadReservations = function() {
            $cordovaProgress.showSimple(true);

            SureSpotApiService.Reservations_GetPrepayReservations(user.userId).then(function(data) {
                console.log(data);
                $cordovaProgress.hide();
                
                $scope.state.reservations = _.map(data, function(reservation) {
                    // qr
                    reservation.qr = qr.toDataURL("SSRIS" + reservation.lot_reservation_id + "SSRIE");

                    // barcode
                    reservation.barcode = textToBase64Barcode('' + reservation.lot_reservation_id);

                    return reservation;
                });
            }, function(err) {
                $cordovaProgress.hide();
                $cordovaDialogs.alert('Please check your network connection.', 'Warning', 'Close');
            });
        };
        
        $scope.loadReservations();

        $scope.getStatus = function(reservation) {
            $cordovaProgress.showSimple(true);

            SureSpotApiService.Reservations_GetStatus(reservation.lot_reservation_id).then(function(data) {
                console.log(data);
                $cordovaProgress.hide();
            }, function(err) {
                $cordovaProgress.hide();
                $cordovaDialogs.alert('Please check your network connection.', 'Warning', 'Close');
            });
        };

        $scope.cancel = function(reservation) {
            $cordovaProgress.showSimple(true);

            SureSpotApiService.Reservations_CancelReservation(reservation.lot_reservation_id).then(function(data) {
                console.log(data);
                $cordovaProgress.hide();

                // reload reservations
                $scope.loadReservations();
            }, function(err) {
                $cordovaProgress.hide();
                $cordovaDialogs.alert('Please check your network connection.', 'Warning', 'Close');
            });
        };
    })
    
    .controller('ReminderSettingsCtrl', function($scope, $state, $stateParams, $cordovaProgress, SureSpotApiService) {
            
    })

    .controller('CardNewCtrl', function($scope, $state, $stateParams, $cordovaProgress, SureSpotApiService) {
        $scope.goMore = function() {
            $state.go('tab.more');
        };

        $scope.state = {
            card: {
                name: '',
                number: '',
                expiry: '',
                cvc: ''
            },

            cardPlaceholders: {
                name: 'Your Full Name',
                number: 'xxxx xxxx xxxx xxxx',
                expiry: 'MM/YY',
                cvc: 'xxx'
            },

            cardMessages: {
                validDate: 'valid\nthru',
                monthYear: 'MM/YYYY'
            },

            cardOptions: {
                debug: false,
                formatting: true
            }
        };

        // load card
        var _card = localStorage.getItem('card');

        if (_card) {
            var card = JSON.parse(_card);
            $scope.state.card = card;
        }

        $scope.onChange = function() {
            console.log('onChange', $scope.state.card);
        };

        $scope.saveCard = function() {
            console.log('saveCard');
            localStorage.setItem('card', JSON.stringify($scope.state.card));
        };
    })

    .controller('MoreCtrl', function($scope, $state, $stateParams, $ionicHistory, $cordovaProgress, SureSpotApiService) {
        $scope.goReminder = function() {
            $state.go('tab.reminder-settings');
        };

        $scope.goCard = function() {
            $state.go('tab.new-card');
        };

        $scope.handleSignOut = function() {
            localStorage.removeItem('user');
            $ionicHistory.clearHistory();
            $state.go('signin');
        };
    })
;
