/*
Documentation: http://surespot1.cloudapp.net/WS_Test/Help

In order to use web services you will have to use the following URL 
    URL: "http://surespot1.cloudapp.net/api/"
Next parameter is the name of Controller
    "http://surespot1.cloudapp.net/ws_test/api/Authentication/"
Last parameter is the name of the method
    validation_login

There is already a method to authenticate, you can use:
    "http://surespot1.cloudapp.net/api/Authentication/validation_login/
    Parameters = username={username}&password={password}
    Type: POST
*/

angular.module('starter.services', [])
    .factory('SureSpotApiService', function($q, $http) {
        return {
            Authentication_SignUp: function(userAccount) {
                /*
                POST api/Authentication/SignUp?firstName={firstName}&lastName={lastName}&userName={userName}&email={email}&password={password}&gender={gender}

                userAccount.firstName;
                userAccount.lastName;
                userAccount.email;
                userAccount.password;
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/Authentication/SignUp?' +
                    'firstName=' + userAccount.firstName + '&' +
                    'lastName=' + userAccount.lastName + '&' +
                    'email=' + userAccount.email + '&' +
                    'password=' + userAccount.password
                );
                
                $http.post(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Authentication_SignIn: function(userAccount) {
                /*
                POST api/Authentication/SignIn?userNameOrEmail={userNameOrEmail}&password={password}

                userAccount.email;
                userAccount.password;
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/Authentication/SignIn?' +
                    'userNameOrEmail=' + userAccount.email + '&' +
                    'password=' + userAccount.password
                );
                
                $http.post(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Authentication_LostPassword: function(userAccount) {
                /*
                POST api/Authentication/LostPassword?userNameOrEmail={userNameOrEmail}&password={password}
                
                userAccount.email;
                userAccount.password;
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/Authentication/LostPassword?' +
                    'userNameOrEmail=' + userAccount.email + '&' +
                    'password=' + userAccount.password
                );
                
                $http.post(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            User_GetCarParks: function(UserId) {
                /*
                GET api/User/GetCarParks?UserId={UserId}
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/User/GetCarParks?' +
                    'UserId=' + UserId
                );
                
                $http.get(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Lots: function(car_park_id) {
                /*
                GET api/Lots?car_park_id={car_park_id}
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/Lots?car_park_id=' + car_park_id
                );
                
                $http.get(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Lots_GetParkingLotsList: function(search) {
                /*
                GET api/Lots/GetParkingLotsList?search={search}
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/Lots/GetParkingLotsList?search=' + search
                );
                
                $http.get(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Lots_GetParkingLot: function(car_park_id) {
                /*
                GET api/Lots/GetParkingLotsList?car_park_id={car_park_id}
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/Lots/GetParkingLotsList?car_park_id=' + car_park_id
                );
                
                $http.get(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Reservations_AddPrepay: function(UserId, car_park_zone_id, datetime_from, datetime_to) {
                /*
                POST api/Reservations/AddPrepay?UserId={UserId}&car_park_zone_id={car_park_zone_id}&datetime_from={datetime_from}&datetime_to={datetime_to}
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/Reservations/AddPrepay?' +
                    'UserId=' + UserId + '&' +
                    'car_park_zone_id=' + car_park_zone_id + '&' +
                    'datetime_from=' + datetime_from + '&' +
                    'datetime_to=' + datetime_to
                );
                
                $http.post(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Payments_PayReservation: function(reservationId, amountToPay, magStrip) {
                /*
                POST api/Payments/PayReservation?reservationId={reservationId}&amountToPay={amountToPay}&magStrip={magStrip}
                */

                var d = $q.defer();
                
                var uri = (
                    '/Payments/PayReservation?' +
                    'reservationId=' + reservationId + '&' +
                    'amountToPay=' + amountToPay + '&' +
                    'magStrip=' + magStrip
                );
                
                // var url = paymentApiUrl + encodeURI(uri);
                var url = paymentApiUrl + uri;
                
                $http.post(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Payments_PayReservationWithCash: function(reservationId, amountToPay) {
                /*
                POST api/Payments/PayReservationWithCash?reservationId={reservationId}&amountToPay={amountToPay}
                */

                var d = $q.defer();
                
                var uri = (
                    '/Payments/PayReservationWithCash?' +
                    'reservationId=' + reservationId + '&' +
                    'amountToPay=' + amountToPay
                );
                
                // var url = paymentApiUrl + encodeURI(uri);
                var url = paymentApiUrl + uri;

                $http.post(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Reservations_GetStatus: function(reservationId) {
                /*
                GET api/Reservations/GetStatus?reservationId={reservationId}
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/Reservations/GetStatus?' +
                    'reservationId=' + reservationId
                );
                
                $http.get(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Reservations: function(UserId) {
                /*
                POST api/Reservations?UserId={UserId}&car_park_zone_id={car_park_zone_id}&datetime_from={datetime_from}&datetime_to={datetime_to}
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/Reservations?' +
                    'UserId=' + UserId
                );
                
                $http.post(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Reservations_GetPrepayReservations: function(UserId) {
                /*
                GET api/Reservations/GetPrepayReservations?status={status}&From={From}&To={To}&car_park_id={car_park_id}&UserId={UserId}
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/Reservations/GetPrepayReservations?' +
                    'status=' + '&' +
                    'From=' + '&' +
                    'To=' + '&' +
                    'car_park_id=' + '&' +
                    'UserId=' + UserId
                );
                
                $http.get(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            Reservations_CancelReservation: function(reservationId) {
                /*
                POST api/Reservations/CancelReservation?reservationId={reservationId}
                */

                var d = $q.defer();
                
                var url = (
                    apiUrl +
                    '/Reservations/CancelReservation?' +
                    'reservationId=' + reservationId
                );
                
                $http.post(url)
                    .then(function(res) {
                        d.resolve(res.data);
                    });
                
                return d.promise;
            },
            encrypt: function(plaintext, key) {
                var newPlaintext = CryptoJS.enc.Utf8.parse(plaintext);

                var newKey = CryptoJS.MD5(key).toString();
                newKey = newKey + newKey.slice(0, 16);
                newKey = CryptoJS.enc.Hex.parse(newKey);
                
                var r = CryptoJS.TripleDES.encrypt(newPlaintext, newKey, {
                    mode: CryptoJS.mode.ECB,
                    padding: CryptoJS.pad.Pkcs7,
                });

                var ciphertext = r.toString();
                ciphertext = encodeURIComponent(ciphertext);
                return ciphertext;
            },
        };
    });
